#include "BuildPyramids.h"

int main(int argc, char** argv)
{
  // Process arguments and get the data file  
  char* dataPath;
  char* dataFile;

  cout << "BuildPyramidsRasterDataset - " 
       << "Environmental Systems Research Institute" << endl << endl;
  
  if (argc != 3)
    {
      cout << "Usage: BuildPyramidsRasterDataset [Path] [Name]" << endl;

      return 0;
    }

  dataPath = argv[1];
  dataFile = argv[2];
  
  InitializeApp();
    
  BuildPyramids(dataPath, dataFile);

  ShutdownApp();

  return 0;
}

HRESULT BuildPyramids(char* dataPath, char* dataFile)
{
  // Get the raster workspace through the workspace factory.
  IWorkspaceFactoryPtr ipRasWkspFac(CLSID_RasterWorkspaceFactory);
  IWorkspacePtr ipWksp;
  IRasterWorkspacePtr ipRasWksp(CLSID_RasterWorkspace);
  IRasterDatasetPtr ipRasDataset;
  IRasterPyramidPtr ipRasPyr;
  
  ipRasWkspFac->OpenFromFile(CComBSTR(dataPath), 0, &ipWksp);
  
  ipRasWksp = ipWksp;    // Query Interface
  ipRasWksp->OpenRasterDataset(CComBSTR(dataFile), &ipRasDataset);
  
  ipRasPyr = ipRasDataset;   // Query Interface
  
  // If there are not already pyramids, create them
  short pyrPresent;
  ipRasPyr->get_Present(&pyrPresent);
  if (!pyrPresent)
    {
      cout << "... working ..." << endl;
      ipRasPyr->Create();
      cout << "Done creating pyramids on " << dataFile << endl;
    }
  else
    {
      cout << "Pyramids already exist for: " << dataFile << endl;
    }
  return S_OK;
}
