#include "LicenseUtilities.h"

// Initialize the application and check out a license if needed.
HRESULT InitializeApp(esriLicenseExtensionCode license)
{
  HRESULT hr;
  
  ::AoInitialize(NULL);
  
  if (license == NULL)
    {
      IAoInitializePtr ipInit(CLSID_AoInitialize);
      esriLicenseStatus status = esriLicenseFailure;
      
      // Try to init as engine 
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeEngine, &status);
      if (status == 10)     // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeEngine, &status);
		  if (status != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  else
			return hr;
		}
      
      // Try to init as ArcView
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcView, &status);
      if (status == 10)     // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcView, &status);
	      if (status != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  else
			return hr;
		}
      
      // Try to init as ArcEditor
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcEditor, &status);
      if (status == 10)     // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcEditor, &status);
		  if (status != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." << std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  else
			return hr;
		}
      
      // Try to init as ArcInfo      
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcInfo, &status);
      if (status == 10)     // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcInfo, &status);
		  if (status != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." << std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  else
			return hr;	    
		}
      
      // No appropriate license is available
      std::cerr << "LicenseUtilities::InitializeApp -- " 
		<< "Unable to initialize ArcObjects "
		<< "(no appropriate license available)." << std::endl;
      return E_FAIL;
    }
  else  // There is a license to check out
    {
      IAoInitializePtr ipInit(CLSID_AoInitialize);
      esriLicenseStatus statusLic = esriLicenseFailure;
      esriLicenseStatus statusExt = esriLicenseFailure;
      
      // Check if have engine and appropriate license
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeEngine, &statusLic);
      ipInit->IsExtensionCodeAvailable(esriLicenseProductCodeEngine, 
				       license, &statusExt);
      if (statusLic == 10 && statusExt == 10)    // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeEngine, &statusLic);
		  if (statusLic != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
	  
		  hr = ipInit->CheckOutExtension(license, &statusExt);
		  if (statusExt != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  return hr;
		}
      
      // Check if have ArcView and appropriate license
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcView, 
				     &statusLic);
      ipInit->IsExtensionCodeAvailable(esriLicenseProductCodeArcView, 
				       license, &statusExt);
      if (statusLic == 10 && statusExt == 10)    // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcView, &statusLic);
		  if (statusLic != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
	  
		  hr = ipInit->CheckOutExtension(license, &statusExt);
		  if (statusExt != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  return hr;
		}
      
      // Check if have ArcEditor and appropriate license
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcEditor, 
				     &statusLic);
      ipInit->IsExtensionCodeAvailable(esriLicenseProductCodeArcEditor,
				       license, &statusExt);
      if (statusLic == 10 && statusExt == 10)   // esriLicenseAvailable
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcEditor, &statusLic);
		  if (statusLic != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
	  
		  hr = ipInit->CheckOutExtension(license, &statusExt);
		  if (statusExt != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  return hr;
		}
      
      // Check if have ArcInfo and appropriate license
      ipInit->IsProductCodeAvailable(esriLicenseProductCodeArcInfo,
				     &statusLic);
      ipInit->IsExtensionCodeAvailable(esriLicenseProductCodeArcInfo,
				       license, &statusExt);
      if (statusLic == 10 && statusExt == 10) // esriLicenseAvail.
		{
		  hr = ipInit->Initialize(esriLicenseProductCodeArcInfo, &statusLic);
		  if (statusLic != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
	  
		  hr = ipInit->CheckOutExtension(license, &statusExt);
		  if (statusExt != esriLicenseCheckedOut)
			{
			  // No appropriate licensing
			  std::cerr << "LicenseUtilities::InitializeApp -- License error." 
					<< std::endl;
			  if (FAILED(hr))
				return hr;
			  else
				return E_FAIL;
			}
		  return hr;
		}
      
      // No appropriate licensing
      std::cerr << "Unable to initialize ArcObjects and check " << std::endl;
      std::cerr << "out the license." << std::endl;
      return E_FAIL;
    }
  
  return E_FAIL;
}

// Shutdown the application and check in the license if needed.
HRESULT ShutdownApp(esriLicenseExtensionCode license)
{
  HRESULT hr;

  // Scope ipInit so released before AoUninitialize call
  {
    IAoInitializePtr ipInit(CLSID_AoInitialize);
    esriLicenseStatus status;
    if (license != NULL)
      {
		hr = ipInit->CheckInExtension(license, &status);
		if (FAILED(hr) || status != 80)  // 80 = esriLicenseCheckedIn
		  std::cerr << "License Check-in failed." << std::endl;
      }
    hr = ipInit->Shutdown();
  }
  
  ::AoUninitialize();
  return hr;
}

// Initialize a Motif ArcGIS Engine Application
HRESULT InitializeEngineMotifApp()
{
  HRESULT hr = E_FAIL;

  ::AoInitialize(NULL);
  {
    IAoInitializePtr ipInit(CLSID_AoInitialize);
    if (ipInit == 0)
      {
        std::cerr << "Licensing Problem: "
                  << "Unable to initialize. This application cannot run!"
		  << std::endl;
        ::AoUninitialize();
        return E_FAIL;
      }
    
    // Determine if the product is available
    esriLicenseStatus status;
    ipInit->IsProductCodeAvailable(esriLicenseProductCodeEngine, &status);
    if (status == esriLicenseAvailable)
      {
        hr = ipInit->Initialize(esriLicenseProductCodeEngine, &status);
        if (status == esriLicenseFailure || FAILED(hr))
          {
            std::cerr << "Licensing Problem: " 
		      << "The initialization failed. "
		      << "This application cannot run!" 
		      << std::endl;
            ShutdownEngineMotifApp();
            return E_FAIL;
          }
      }
    else 
      {
        std::cerr << "Licensing Problem: "
		  << "The ArcGIS Engine product is unavailable. "
		  << "This application cannot run!"
		  << std::endl;  
        ::AoUninitialize();
        return E_FAIL;
      }
  }
  
  return hr;
}

HRESULT ShutdownEngineMotifApp()
{
  HRESULT hr;

  {
    IAoInitializePtr ipInit(CLSID_AoInitialize);
    hr = ipInit->Shutdown();
  }
  ::AoUninitialize();

  return hr;
}
